# Label Help

The Label Help module provides a way of adding help text to Drupal form fields
which appears below the form label but above the form field itself.

For a full description of the module, visit the
[project page](https://www.drupal.org/project/label_help).

Submit bug reports and feature suggestions, or track changes in the
[issue queue](https://www.drupal.org/project/issues/label_help).

## Requirements

This module requires no modules outside of Drupal core.


## Installation

1. Install and enable as you would normally install a contributed Drupal module.
   For further information, see [Installing Drupal Modules](https://www.drupal.org/docs/extending-drupal/installing-drupal-modules).

2. When editing a form field using Drupal's Field UI user interface for the
   Field API, you'll see an option named 'Label help message' that lets you
   enter help text which will appear below the label for that field.

3. This module does _not_ affect the placement of the 'Help text' field that
   normally goes _below_ the form field. You could therefore have separate
   help text appearing in both places. Most people will probably want to leave
   put text in one place and leave the other option blank.


## Troubleshooting

1.  Label Help placement logic 'use case' debugging.

    If a given field or widget type is not behaving as expected, enable debug
    mode in settings.php with the following line:

        $settings['label_help_debug'] = TRUE;

    Then reload the form page to see debug output identifying the field widget
    type and use case matched by Label Help placement logic. If no use case is
    detected you'll see a warning message.

2.  Form API dump debugging

    To troubleshoot Form API structures for a given form field enable the Form
    API element dump mode in settings.php with the following line:

        $settings['label_help_debug_dump'] = TRUE;

    This debug mode leverages the Symfony Variable Dumper `dump()` to provide
    a drillable screen dump of the Form API element array. However, because
    this dump runs inside the form_alter, it can result in incomplete page
    load. This is expected.

## Using Label Help to create form fields programmatically

The Label Help field defines a theme option named 'description at top' which
can be used to insert label help text in form fields that are defined
programmatically. The following function would therefore define a form with
label help text at the top of field 'example':

```php
function mymodule_form($form, &$form_state) {
  $form['example'] = [
    '#type' => 'textfield',
    '#title' => t('Example'),
    '#theme_options' => [
      'description at top' => t('Label help text for the example field.'),
    ],
  ];
  return $form;
}
```

## Modifying form fields using hook_form_alter()

Drupal's hook_form_alter() functions may be used in a custom module or theme
to add Label Help text to existing form fields. For example, the following
function uses hook_form_user_login_alter() to change Drupal's user login form
so that it displays label text top of field 'example':

```php
function mymodule_form_user_login_alter(&$form, &$form_state, $form_id) {
  $form['name']['#theme_options'] = [
    'description at top' => t('Enter your @s username.', [
      '@s' => variable_get('site_name', 'Drupal')
    ]),
  ];
  unset($form['name']['#description']);
  $form['pass']['#theme_options'] = [
    'description at top' => t(
      'Enter the password that accompanies your username.'
    ),
  ];
  unset($form['pass']['#description']);
}
```

## Contributing

1.  DDEV configuration is available.

    https://label-help.ddev.site is available after `ddev start`.

2.  The DDEV add-on https://github.com/ddev/ddev-drupal-contrib is available.

    Setup steps:

        git clone git@git.drupal.org:project/label_help.git && cd label_help
        ddev start
        ddev poser
        ddev symlink-project
        ddev drush site:install
        ddev drush user:login

3.  Ensure code quality before submitting MRs.

        ddev phpcs
        ddev phpstan

    Automatically fix php syntax errors with `phpcbf`:

        ddev phpcbf

    This avoids wait time for the Drupal GitLabCI build to test code quality.
